@@ -23,6 +23,137 @@ $(function() {
 		}
 	}
 
+	// Editors hash object
+	var editors = {};
+
+	var tabId = 1;
+	window.AddEditorTab = function(tabName) {
+		
+		if(!tabName) {
+			// Tab name
+			tabName = "Untitled " + tabId;
+		}
+		
+		// Generate editor id
+		editorId = "editor" + tabId;
+
+		// Add the tab component
+		$("#myTab").append('<li><a href="#' + tabId + '" data-toggle="tab">' + tabName + '</a></li>');
+
+		// Add the tab editor
+		$("#myTabContent").append(
+			'<div class="tab-pane" id="' + tabId + '">' +
+				'<div class="span6 farabi-bordered" id="editor-border">' +
+					'<b>Editor:</b>' +
+					'<textarea id="'+editorId+'" name="' + editorId +'" class="editor"></textarea>' +
+					'<div class="editor_stats" class="pull-right"></div>' +
+				'</div>' +
+				'<div class="span6 farabi-bordered">' +
+					'<b>POD  Preview:</b>' + 
+					'<div  id="pod-tab">' +
+					'</div>' +
+				'</div>' +
+			'</div>');
+		
+		// Create the CodeMirror editor instance
+		var editor = CodeMirror.fromTextArea(document.getElementById(editorId), {
+			//TODO these should be configurable!
+			lineNumbers: true,
+			matchBrackets: true,
+			tabSize: 4,
+			indentUnit: 4,
+			indentWithTabs: true,
+			extraKeys: {
+				"F1": function(cm) {
+					displayHelp(cm);
+				},
+				'Alt-A': function(cm) {
+					$(".actions-button").click();
+				}
+			}
+		});
+
+		// Hook up with cursor activity event
+		editor.on("cursorActivity", function(cm) {
+				// Highlight active line
+				cm.removeLineClass(hlLine, "background", "activeline");
+				hlLine = cm.addLineClass(cm.getCursor().line, "background", "activeline");
+
+				// Highlight selection matches
+				cm.matchHighlight("CodeMirror-matchhighlight");
+
+				// Show editor statistics
+				showEditorStats(cm);
+		});
+
+		// Hook up with cursor activity event
+		var timeoutId;
+		editor.on("change", function(cm) {
+				clearInterval(timeoutId);
+				timeoutId = setTimeout(function() {
+					var mode = cm.getOption("mode");
+					if(mode == "perl") {
+						podChanged(cm);
+						podCheck();
+					} else {
+						$("#pod-tab").html('Not supported for mode "' + mode + '"');
+					}
+				}, 250);
+		});
+
+		
+		// Store editor reference
+		editors[editorId] = editor;
+		
+		// This is the current editor
+		currentEditor = editor;
+
+		// Show the last editor tab which we added
+		var $lastTab = $("#myTab a:last");
+		$lastTab.tab('show');
+		
+		// And refresh the editor once it is shown
+		$lastTab.on('shown', function (e) {
+			editor.refresh();
+		});
+
+		// Run these when we exit this one
+		setTimeout(function() {
+			// Editor is by default Perl
+			ChangeMode(editor, 'perl');
+
+			// focus!
+			editor.focus();
+
+			// Show editor stats at startup
+			showEditorStats(editor);
+
+			// Trigger theme selection
+			$("#theme_selector").change();
+
+			var mode = editor.getOption("mode");
+			if(mode == "perl") {
+				// Update POD tab at startup
+				podChanged(editor);
+
+				// Run Pod::Checker at startup
+				podCheck();
+			} else {
+				$("#pod-tab").html('Not supported for mode "' + mode + '"');
+			}
+
+		}, 0);
+
+		// Highlight active line
+		var hlLine = editor.addLineClass(0, "background", "activeline");
+		
+		tabId++;
+	};
+	
+
+	// Add an empty one editor tab on startup
+	AddEditorTab();
+
 	function _displayHelp(topic, bShowDialog) {
 		$.post('/help_search', {"topic": topic}, function(results) {
 			if(results.length > 0) {
@@ -58,7 +189,7 @@ $(function() {
 		CodeMirror.autoLoadMode(cm, modeFile);	
 	}
 
-	function plural(number) {
+	var plural = function(number) {
 		if(number == 1) {
 			return 'st';
 		} else if(number == 2) {
@@ -68,12 +199,12 @@ $(function() {
 		}
 	}
 
-	function showEditorStats(cm) {
+	var showEditorStats = function(cm) {
 		var cursor = cm.getCursor();
 		var selection = cm.getSelection();
 		var line_number = cursor.line + 1;
 		var column_number = cursor.ch + 1;
-		$('#editor_stats').html(
+		$('.editor_stats').html(
 			'<strong>' + line_number + '</strong>' + plural(line_number) + ' line' +
 			', <strong>' + column_number + '</strong>' + plural(column_number) + ' column' +
 			', <strong>' + cm.lineCount() + '</strong>&nbsp;lines' +
@@ -89,86 +220,12 @@ $(function() {
 		});
 	};
 
-	var editor = CodeMirror.fromTextArea(document.getElementById(editorId), {
-		lineNumbers: true,
-		matchBrackets: true,
-		tabSize: 4,
-		indentUnit: 4,
-		indentWithTabs: true,
-		extraKeys: {
-			"F1": function(cm) {
-				displayHelp(cm);
-			},
-			'Alt-A': function(cm) {
-				$(".actions-button").click();
-			}
-		}
-	});
-
 	/** Get the current editor */
 	window.GetCurrentEditor = function() {
 		//TODO return a better one :)
-		return editor;
+		return currentEditor;
 	};
 	
-	// Hook up with cursor activity event
-	editor.on("cursorActivity", function(cm) {
-			// Highlight active line
-			cm.removeLineClass(hlLine, "background", "activeline");
-			hlLine = cm.addLineClass(cm.getCursor().line, "background", "activeline");
-
-			// Highlight selection matches
-			cm.matchHighlight("CodeMirror-matchhighlight");
-
-			// Show editor statistics
-			showEditorStats(cm);
-	});
-
-	// Hook up with cursor activity event
-	var timeoutId;
-	editor.on("change", function(cm) {
-			clearInterval(timeoutId);
-			timeoutId = setTimeout(function() {
-				var mode = cm.getOption("mode");
-				if(mode == "perl") {
-					podChanged(cm);
-					podCheck();
-				} else {
-					$("#pod-tab").html('Not supported for mode "' + mode + '"');
-				}
-			}, 250);
-	});
-
-	// Run these when we exit this one
-	setTimeout(function() {
-		// Editor is by default Perl
-		ChangeMode(editor, 'perl');
-
-		// focus!
-		editor.focus();
-
-		// Show editor stats at startup
-		showEditorStats(editor);
-
-		// Trigger theme selection
-		$("#theme_selector").change();
-
-		var mode = editor.getOption("mode");
-		if(mode == "perl") {
-			// Update POD tab at startup
-			podChanged(editor);
-
-			// Run Pod::Checker at startup
-			podCheck();
-		} else {
-			$("#pod-tab").html('Not supported for mode "' + mode + '"');
-		}
-
-	}, 0);
-
-	// Highlight active line
-	var hlLine = editor.addLineClass(0, "background", "activeline");
-
 	$(".results").hide();
 
 	$(document).on('action-perl-doc', function() {
@@ -177,6 +234,7 @@ $(function() {
 
 	$(document).on('action-perl-tidy', function() {
 		var $output = $('#output');
+		var editor = GetCurrentEditor();
 		$.post('/perl-tidy', {"source": editor.getValue()}, function(data) {
 			if(data.error == '') {
 				editor.setValue(data.source);
@@ -208,6 +266,7 @@ $(function() {
 	var perlCheckerWidgets = [];
 
 	var podCheck = function() {
+		var editor = GetCurrentEditor();
 		$.post('/pod-check', {"source": editor.getValue()}, function(problems) {
 			for (var i = 0; i < perlCheckerWidgets.length; i++) {
 				editor.removeLineWidget(perlCheckerWidgets[i].widget);
@@ -257,7 +316,7 @@ $(function() {
 	});
 
 	var onCloseFocusEditor = function () {
-		editor.focus();
+		GetCurrentEditor().focus();
 	};
 
 	// All modal dialogs are now hidden on startup
@@ -281,5 +340,9 @@ $(function() {
 	}).change(function() {
 		_displayHelp($(this).val(), true);
 	});;
+	
+	$(document).on('action-new-file', function() {
+		AddEditorTab();
+	});
 
 });
\ No newline at end of file
